import os
from datetime import datetime, timezone, timedelta
from flask import Flask, render_template, send_from_directory, request, jsonify
from modules import awsinteract

CUR_DIR = os.path.dirname(os.path.abspath(__file__))
app = Flask(__name__)

awsinteract.init_aws()
dynamodb = awsinteract.DynamoDB()

JST = timezone(timedelta(hours=+9))

def _get_timestamp_from_ttl(ttl):
    return (datetime.fromtimestamp(int(ttl), JST) - timedelta(days=1)).strftime('%Y-%m-%d %H:%M:%S')

def _get_identifiers(scan_data):
    return list(set([a.get('identifier') for a in scan_data]))

def _create_data():
    scan_data = dynamodb.scan_table()
    scan_data.reverse()
    cameras = _get_identifiers(scan_data)
    if len(cameras) == 0:
        return None

    res = {'Cameras': []}
    for camera in cameras:
        c = {'name': camera, 'data': []}
        res['Cameras'].append(c)

    for s in scan_data:
        d = {'url': s['url'], 'timestamp': _get_timestamp_from_ttl(s['ttl'])}
        for i, camera in enumerate(res['Cameras']):
            if s['identifier'] == camera['name']:
                res['Cameras'][i]['data'].append(d)
                break

    return res

@app.route("/")
def mainpage():
    data = _create_data()        
    return render_template("index.html",
                           title="Armadillo Camera",
                           results=data
    )

@app.route("/_scantable")
def testpage():
    data = _create_data()        
    return jsonify(data)

@app.route("/robots.txt")
def google():
    return send_from_directory(app.static_folder, request.path[1:])

if __name__ == '__main__':
    app.run(host='0.0.0.0')
