import os
import time
import datetime
import threading
import cv2
import subprocess
from modules import ledhandler

def get_list_of_cameras():
    try:
        cmd = 'ls -l /dev/v4l/by-id'
        res = subprocess.check_output(cmd.split())
        by_id = res.decode()
    except:
        return

    devices = []
    for line in by_id.split('\n'):
        if '../../video' in line:
            device = {}
            tmp = __split(line, ' ')
            if '' in tmp:
                tmp.remove('')
            name = tmp[8]
            deviceId = tmp[10].replace('../../video', '')
            device['cameraId'] = deviceId
            device['cameraName'] = name
            devices.append(device)

    return devices

def __split(string, val):
        tmp = string.split(val)
        if('' in tmp):
            tmp.remove('')
        return tmp

class Camera:
    def __init__(self, period, save_path, **cam_info):
        self.cameraId = int(cam_info['cameraId'])
        self.cameraName = cam_info['cameraName']
        self.period = period
        self.save_path = os.path.join(save_path, self.cameraName)

    def _take_picture(self):
        cap = cv2.VideoCapture(self.cameraId)

        ret, frame = cap.read()
        cv2.imwrite(
            os.path.join(
                self.save_path,
                datetime.datetime.now().strftime('%Y%m%d%H%M%S') + '.jpg'
            ),
            frame
        )

    def start_take_picture(self):
        while True:
            time.sleep(self.period)
            try:
                self._take_picture()
            except:
                ledhandler.camera_error()

    def get_save_path(self):
        return self.save_path
