#!/bin/sh

. /images/install.conf

PATH=/bin:/sbin:/usr/bin:/usr/sbin

#
# internal parameters
#
DEV_EMMC=/dev/mmcblk0
DEV_SD=/dev/mmcblk1
FIRST_PARTITION_SD=${DEV_SD}p1
BOOTLOADER_EMMC_PATH=${DEV_EMMC}
BOOTLOADER_EMMC_FORCERO_PATH=/sys/block/mmcblk0boot0/force_ro
RESERVED1_EMMC_PATH=${DEV_EMMC}p1
IMAGES_EMMC_PATH=${DEV_EMMC}p2
RESERVED2_EMMC_PATH=${DEV_EMMC}p3
USER_PART_EMMC_PATH=${DEV_EMMC}p4
BOOTLOADER_PARM_SIZE_K=512
BOOTLOADER_PARM_OFFSET_K=512
STEP=0
LED_CLASS_PATH=/sys/class/leds
NVMEM_PATH=/sys/bus/nvmem/devices/imx-ocotp0/nvmem

#
# progress led functions
#
progress_running=:
progress_ok=:
progress_ng=:

redirect_if_exist() {
	if [ -f "$2" ]; then
		echo "$1" > "$2"
	fi
}

led_control() {
	local LED=$1
	local MODE=$2
	local INTERVAL=250
	if [ ${MODE} == "off" ]; then
		redirect_if_exist none ${LED_CLASS_PATH}/${LED}/trigger
		redirect_if_exist 0 ${LED_CLASS_PATH}/${LED}/brightness
	elif [ ${MODE} == "on" ]; then
		redirect_if_exist none ${LED_CLASS_PATH}/${LED}/trigger
		redirect_if_exist 1 ${LED_CLASS_PATH}/${LED}/brightness
	elif [ ${MODE} == "blink" ]; then
		redirect_if_exist timer ${LED_CLASS_PATH}/${LED}/trigger
		redirect_if_exist ${INTERVAL} ${LED_CLASS_PATH}/${LED}/delay_off
		redirect_if_exist ${INTERVAL} ${LED_CLASS_PATH}/${LED}/delay_on
	fi
}

progress_running_a640() {
	led_control "green"  "blink"
	led_control "red"    "off"
	led_control "yellow" "off"
}

progress_ok_a640() {
	led_control "green"  "on"
	led_control "red"    "on"
	led_control "yellow" "on"
}

progress_ng_a640() {
	led_control "green"  "off"
	led_control "red"    "blink"
	led_control "yellow" "off"
}

progress_running_a610() {
	led_control "green"  "off"
	led_control "red"    "off"
	led_control "yellow" "blink"
}

progress_ok_a610() {
	led_control "green"  "on"
	led_control "red"    "on"
	led_control "yellow" "on"
}

progress_ng_a610() {
	led_control "green"  "off"
	led_control "red"    "off"
	led_control "yellow" "off"
}

print_serial() {
	echo $*
}

print_step() {
	STEP=$(($STEP+1))
	print_serial ""
	print_serial ""
	print_serial "=== Step.${STEP} $* ==="
	print_serial ""
}

error_exit() {
	print_serial "##### $*"
	${progress_ng}
	exit 2
}

emmc_format() {
	sfdisk /dev/mmcblk0 <<EOF
# partition table of /dev/mmcblk0
unit: sectors

/dev/mmcblk0p1 : start=     2048, size=    62592, Id= b
/dev/mmcblk0p2 : start=        0, size=        0, Id= 0
/dev/mmcblk0p3 : start=    64640, size=   250048, Id= b
/dev/mmcblk0p4 : start=        0, size=        0, Id= 0
EOF

	if [ -n "$USER_PART_SIZE" ]; then
#fdisk -u -l /dev/mmcblk0
#
# Disk /dev/mmcblk0: 3791 MB, 3791650816 bytes
# 4 heads, 16 sectors/track, 115712 cylinders, total 7405568 sectors
# Units = sectors of 1 * 512 = 512 bytes
#
#         Device Boot      Start         End      Blocks   Id  System
# /dev/mmcblk0p1            2048       64639       31296    b  W95 FAT32
# /dev/mmcblk0p2         2411840     7405567     2496864   83  Linux
# /dev/mmcblk0p3           64640      314687      125024    b  W95 FAT32
# /dev/mmcblk0p4          314688     2411839     1048576   83  Linux
#
# Partition table entries are not in disk order
        fdisk /dev/mmcblk0 <<EOF
n
p
4

${USER_PART_SIZE}
n
p


t
4
83
t
2
83
p
w
EOF
	else
# fdisk -l -u /dev/mmcblk0
#
# Disk /dev/mmcblk0: 3791 MB, 3791650816 bytes
# 4 heads, 16 sectors/track, 115712 cylinders, total 7405568 sectors
# Units = sectors of 1 * 512 = 512 bytes
#
#         Device Boot      Start         End      Blocks  Id System
# /dev/mmcblk0p1            2048       64639       31296   b Win95 FAT32
# /dev/mmcblk0p2          314688     7405567     3545440  83 Linux
# /dev/mmcblk0p3           64640      314687      125024   b Win95 FAT32
#
# Partition table entries are not in disk order
        fdisk /dev/mmcblk0 <<EOF
n
p
2


t
2
83
p
w
EOF
	fi

	mkfs.vfat ${RESERVED1_EMMC_PATH}
	mke2fs -t ext4 ${IMAGES_EMMC_PATH}
	mkfs.vfat ${RESERVED2_EMMC_PATH}
	if [ -n "$USER_PART_SIZE" ]; then
		mke2fs -t ext4 ${USER_PART_EMMC_PATH}
	fi

}

setup_bootloader() {
	if [ ! -z "${BOOTLOADER_IMAGE_MD5}" ]; then
		echo "${BOOTLOADER_IMAGE_MD5}  ${BOOTLOADER_IMAGE}" | md5sum -c
		[ $? -eq 0 ] || return 1
	fi

	if [ -f "${BOOTLOADER_IMAGE}" ]; then
#		echo 0 > ${BOOTLOADER_EMMC_FORCERO_PATH}
		dd if=${BOOTLOADER_IMAGE} of=${DEV_EMMC} bs=1k seek=1
		sync
#		echo 1 > ${BOOTLOADER_EMMC_FORCERO_PATH}
	else
		echo "file not found ${BOOTLOADER_IMAGE}"
		return 1
	fi

	dd if=/dev/zero bs=1k count=${BOOTLOADER_PARM_SIZE_K} | \
	tr "\000" "\377" | dd of=${BOOTLOADER_EMMC_PATH} bs=1k seek=${BOOTLOADER_PARM_OFFSET_K}
	sync
}

extract_file() {
	local DEVICE=$1
	local FS=$2
	local FILE=$3
	local MD5=$4

	if [ ! -z "${FILE}" ]; then
		if [ ! -z "${MD5}" ]; then
			echo "${MD5}  ${FILE}" | md5sum -c
			[ $? -eq 0 ] || return 1
		fi

		mkdir -p /tmp/${DEVICE}
		mount -t ${FS} ${DEVICE} /tmp/${DEVICE}
		tar xzf ${FILE} -C /tmp/${DEVICE}
		sync
		umount /tmp/${DEVICE}
	else
		echo "file not found ${FILE}"
		return 1
	fi
}

copy_file() {
	local DEVICE=$1
	local FS=$2
	local FILE=$3
	local MD5=$4
	local TO=$5

	if [ ! -z "${FILE}" ]; then
		if [ ! -z "${MD5}" ]; then
			echo "${MD5}  ${FILE}" | md5sum -c
			[ $? -eq 0 ] || return 1
		fi

		mkdir -p /tmp/${DEVICE}
		mount -t ${FS} ${DEVICE} /tmp/${DEVICE}
		mkdir -p /tmp/${DEVICE}/${TO%/*}
		cp ${FILE} /tmp/${DEVICE}/${TO}
		ls /tmp/${DEVICE}/${TO}
		sync
		umount /tmp/${DEVICE}
	else
		echo "file not found ${FILE}"
		return 1
	fi
}

copy_etc_from_sd_first_part()
{
	# Since FAT does not have permission, it is copied to the
	# tmp directory once and the permission is given.

	mkdir -p /tmp/${FIRST_PARTITION_SD}
	mkdir -p /tmp/temporary

	mount -t vfat ${FIRST_PARTITION_SD} /tmp/${FIRST_PARTITION_SD}
	if [ -n "$(ls /tmp/${FIRST_PARTITION_SD})" ]; then
		cp -r /tmp/${FIRST_PARTITION_SD}/etc /tmp/temporary/

		find /tmp/temporary/* -type f | xargs chmod 600
		mkdir -p /tmp/${IMAGES_EMMC_PATH}
		mount -t ext4 ${IMAGES_EMMC_PATH} /tmp/${IMAGES_EMMC_PATH}
		cp -r /tmp/temporary/* /tmp/${IMAGES_EMMC_PATH}
		umount /tmp/${IMAGES_EMMC_PATH}
		rm -rf /tmp/${IMAGES_EMMC_PATH}
	fi
	umount /tmp/${FIRST_PARTITION_SD}

	rm -rf \
	   /tmp/temporary \
	   /tmp/${FIRST_PARTITION_SD}
}

protect_mmc() {
	local DEVICE=$1

	local BLOCKS=$(mmc writeprotect user get ${DEVICE} | egrep -o "[0-9]+/[0-9]+" | cut -d '/' -f 1)
	mmc writeprotect user set temp 0 ${BLOCKS} ${DEVICE}
	mmc writeprotect user get ${DEVICE} | grep "Temporary Write Protection"
	[ $? -ne 0 ] && error_exit "eMMC protect error: ${DEVICE}"
}

#
# main
#
run () {
	print_serial "*+*+* Install Start!! *+*+*"
	${progress_running}

	print_step "Formatting eMMC"
		emmc_format
		fdisk -l /dev/mmcblk0

	print_step "Setup Partition"
		extract_file ${IMAGES_EMMC_PATH} ext4 \
			"${ROOTFS_ARCHIVE}" "${ROOTFS_ARCHIVE_MD5}"
		[ $? -ne 0 ] && error_exit "MD5 error: ${ROOTFS_ARCHIVE}"
		copy_file ${IMAGES_EMMC_PATH} ext4 \
			"${KERNEL_IMAGE}" "${KERNEL_IMAGE_MD5}"  "/boot"
		[ $? -ne 0 ] && error_exit "MD5 error: ${KERNEL_IMAGE}"
		copy_file ${IMAGES_EMMC_PATH} ext4 \
			"${DEVICETREE_IMAGE}" "${DEVICETREE_IMAGE_MD5}" "/boot"
		[ $? -ne 0 ] && error_exit "MD5 error: ${DEVICETREE_IMAGE}"
		copy_etc_from_sd_first_part

	print_step "Setup Bootloader"
		setup_bootloader

	print_step "sync"
		sync
	protect_mmc /dev/mmcblk0gp0
	print_serial "*+*+* Install Completed!! *+*+*"

	${progress_ok}
}

get_kotai_code() {
	cat $NVMEM_PATH > /tmp/a
	hexdump -v -s 0xa0 -n 8 -e '/4 "%08X"' /tmp/a
	rm -f /tmp/a
}

send_log() {
# Place holder to send LOGFILE from armadillo to any server
	rm ${LOGFILE}
}

case "$(get_kotai_code)" in
	"00000097"*|"0000009C"*)
		progress_running=progress_running_a640
		progress_ok=progress_ok_a640
		progress_ng=progress_ng_a640
		;;
	"000000B4"*|"000000B7"*)
		progress_running=progress_running_a610
		progress_ok=progress_ok_a610
		progress_ng=progress_ng_a610
		;;
	*)
		error_exit "error: unkown board"
		;;
esac

# there is no way to check that run() failed or not.  we don't have
# pipestatus nor the scope for the global variables.
run 2>&1 | tee ${LOGFILE}
send_log 2>&1 | tee ${SEND_LOGFILE}
if [ -e ${LOGFILE} ]; then
	${progress_ng}
fi
while true; do :; done
